"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdRecord = void 0;
const errors_1 = require("../utils/errors");
const db_1 = require("../utils/db");
const uuid_1 = require("uuid");
class AdRecord {
    constructor(obj) {
        if (!obj.name || obj.name.length > 100) {
            throw new errors_1.ValidationError('Nazwa ogłoszenia nie może być pusta, ani przekraczać 100 znaków.');
        }
        if (obj.description.length > 1024) {
            throw new errors_1.ValidationError('Opis nie może być dłuższy niż 1000 znakow.');
        }
        if (obj.price < 0 || obj.price > 9999999) {
            throw new errors_1.ValidationError('Cena nie może być mniejsza niż zero lub większa niż 9 999 999.');
        }
        if (!obj.url || obj.url.length > 100) {
            throw new errors_1.ValidationError('Adres URL ogłoszenia nie może być pusty, ani przekraczać 100 znaków.');
        }
        if (typeof obj.lat !== "number" || typeof obj.lon !== "number") {
            throw new errors_1.ValidationError('Nie można zlokalizować ogłoszenia.');
        }
        this.id = obj.id;
        this.name = obj.name;
        this.description = obj.description;
        this.url = obj.url;
        this.price = obj.price;
        this.lon = obj.lon;
        this.lat = obj.lat;
        this.clicksCounter = obj.clicksCounter;
        this.secondUrl = obj.secondUrl;
    }
    static getOne(id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield db_1.pool.execute('update `ads` set `clicksCounter` = `clicksCounter` + 1 where `id` = :id', {
                id,
            });
            const [results] = yield db_1.pool.execute('select * from `ads` where `id` = :id', {
                id,
            });
            return results.length === 0 ? null : new AdRecord(results[0]);
        });
    }
    // static async listAll(): Promise<AdRecord[]> {
    //     const [results] = await pool.execute('select * from `ads`') as AdRecordResults;
    //     return results.map((obj) => new AdRecord(obj));
    // }
    //powyżej zbędne pod dodaniu funkcji findAll z wyszukaniem LIKE
    static findAll(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute('select * from `ads` where `name` like :search', { search: `%${name}%`, });
            return results.map(result => {
                const { id, lat, lon, clicksCounter } = result;
                return { id, lat, lon, clicksCounter };
            });
        });
    }
    insert() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.id) {
                this.id = (0, uuid_1.v4)();
            }
            else
                throw new Error('Cannot insert something that is alredy inserted.');
            yield db_1.pool
                .execute('insert into `ads` (`id`, `name`, `description`,`price`, `url`, `lat`, `lon`, `secondUrl`) values (:id, :name, :description, :price, :url, :lat, :lon, :secondUrl)', this
            //     {
            //     id: this.id,
            //     name: this.name,
            //     description: this.description,
            //     price: this.price,
            //     url: this.url,
            //     lat: this.lat,
            //     lon: this.lon,
            // }
            );
            return this.id;
        });
    }
}
exports.AdRecord = AdRecord;
//# sourceMappingURL=ad.record.js.map